import uuid
import logging
from flask import g, request

# Returns the current request ID or a new one if there is none
def request_id():
    if getattr(g, 'request_id', None):
        return g.request_id
    return uuid.uuid4()

class RequestIdFilter(logging.Filter):

    def __init__(self, server, service, *args, **kwargs):
        super(RequestIdFilter, self).__init__(*args, **kwargs)
        self.flaskApp = server
        self.service = service

    # This is a logging filter that makes the request ID available for use in the logging format.
    def filter(self, record, *args, **kwargs):
        # record.request_id = request_id(self.flaskApp) if (self.flaskApp is not None and self.flaskApp.has_request_context()) else ''
        record.request_id = request_id() if (self.flaskApp is not None) else '-'
        # record.request_id = request_id() if (self.service is not None) else '00000000-0000-0000-0000-000000000000'
        record.service = self.service if self.service is not None else 'motifer'
        return super(RequestIdFilter, self).filter(record, *args, **kwargs)


"""Generic logs pattern builder for Python and Flask, built on top of logging module."""
__author__ = "Ankur Mahajan"
__copyright__ = "Copyright 2023"
__version__ = "1.0.0"
__maintainer__ = "Ankur Mahajan"
__email__ = "mahajanankur.nith@gmail.com"

from motifer.logger_factory import LogFactory
from motifer.flask_factory import FlaskLogFactory


import logging

# Logging formatter supporting colorized output
class LogFormatter(logging.Formatter):

    COLOR_CODES = {
        logging.CRITICAL: "\033[1;35m", # bright/bold magenta
        logging.ERROR:    "\033[1;31m", # bright/bold red
        logging.WARNING:  "\033[1;33m", # bright/bold yellow
        logging.INFO:     "\033[0;37m", # white / light gray
        logging.DEBUG:    "\033[1;30m"  # bright/bold black / dark gray
    }

    RESET_CODE = "\033[0m"

    def __init__(self, color, *args, **kwargs):
        super(LogFormatter, self).__init__(*args, **kwargs)
        self.color = color

    def format(self, record, *args, **kwargs):
        if (self.color == True and record.levelno in self.COLOR_CODES):
            record.color_on  = self.COLOR_CODES[record.levelno]
            record.color_off = self.RESET_CODE
        else:
            record.color_on  = ""
            record.color_off = ""
        return super(LogFormatter, self).format(record, *args, **kwargs)


import sys, logging, json, uuid, time
from flask import g, request
from motifer.formatters import LogFormatter
from motifer.filters import RequestIdFilter
# logger = None

class LogFactory:
    service = None
    log_level = logging.INFO
    options = None
    server = None
    console_log_output = 'stdout'
    console_log_color = True
    logfile_file = None
    logfile_log_level = logging.INFO
    logfile_log_color = True
    log_line_template = '%(color_on)s[%(asctime)s] [%(service)s] [%(request_id)s] [%(levelname)-4s] [%(filename)s:%(lineno)d] %(message)s%(color_off)s'

    def __init__(self, service, log_level, server, **options):
        self.service = service or "service-name"
        self.log_level = log_level.upper()
        self.options = options
        self.console_log_output = options.get('console_log_output').lower() if (options is not None and options.get('console_log_output') is not None) else 'stdout'
        # self.console_log_color = True if (options is not None and options.get('console_log_color') is not None) else False
        self.logfile_file = options.get('logfile_file') if (options is not None and options.get('logfile_file') is not None) else None
        self.logfile_log_level = options.get('logfile_log_level').upper() if (options is not None and options.get('logfile_log_level') is not None) else logging.INFO
        self.logfile_log_color = True if (options is not None and options.get('logfile_log_color') is not None) else False
        # Change log level labels in all the formatters
        logging.addLevelName(logging.WARNING, "WARN")
        # logging.addLevelName(logging.CRITICAL, "CRIT")
        logger = logging.getLogger(self.service)
        if(server is not None):
            self.server = server
            @server.before_request
            def before_request():
                g.start_time = time.time()
                g.request_id = str(uuid.uuid4())
                # TIMESTAMP_ISO [request] [REQUEST_ID] [APP_NAME] [LOG_LEVEL] [REQUEST_METHOD] [REQUEST_IP] [API_PATH] [BODY]
                logger.info("[request] [{REQUEST_METHOD}] [{REQUEST_IP}] [{API_PATH}] [{BODY}]".format(REQUEST_METHOD = request.method, REQUEST_IP=request.remote_addr, API_PATH=request.path, BODY=request.form))

            @server.after_request
            def after_request(response):
                response_time = int((time.time() - g.start_time) * 1000)
                # TIMESTAMP_ISO [response] [REQUEST_ID] [APP_NAME] [LOG_LEVEL] [REQUEST_METHOD] [REQUEST_IP] [API_PATH] [RESPONSE_STATUS] [CONTENT_LENGTH] [RESPONSE_TIME] [USER_AGENT]
                logger.info("[response] [{REQUEST_METHOD}] [{REQUEST_IP}] [{API_PATH}] [{RESPONSE_STATUS}] [{CONTENT_LENGTH}] [{RESPONSE_TIME}] [{USER_AGENT}]".format(REQUEST_METHOD=request.method, REQUEST_IP=request.remote_addr, API_PATH=request.path, RESPONSE_STATUS=response.status_code, CONTENT_LENGTH=response.content_length, RESPONSE_TIME=response_time, USER_AGENT=request.user_agent))
                return response

    # Get logger factory.
    def get_logger(self):
        # Create logger
        logger = logging.getLogger(self.service)
        # stop propagting to root logger
        logger.propagate = False
        # Set global log level to 'debug' (required for handler levels to work)
        logger.setLevel(logging.DEBUG)
        # https://stackoverflow.com/questions/30945460/formatting-flask-app-logs-in-json 
        logging.getLogger('gunicorn').propagate = False
        # logging.getLogger('werkzeug').disabled = True
        self.alter_werkzeug_logger()
        # Create console handler
        if (self.console_log_output == "stdout"):
            self.console_log_output = sys.stdout
        elif (self.console_log_output == "stderr"):
            self.console_log_output = sys.stderr
        else:
            print("Failed to set console output: invalid output: '%s'" % self.console_log_output)
            return False
        console_handler = logging.StreamHandler(self.console_log_output)
        # Set console log level
        try:
            console_handler.setLevel(self.log_level) # only accepts uppercase level names
        except:
            print("Failed to set console log level: invalid level: '%s'" % self.log_level)
            return False
            
        requestFilters = RequestIdFilter(server= self.server, service= self.service)
        logger.addFilter(requestFilters)
        # Create and set formatter, add console handler to logger
        console_formatter = LogFormatter(fmt=self.log_line_template, color=self.console_log_color)
        console_handler.setFormatter(console_formatter)
        logger.addHandler(console_handler)
        if(self.logfile_file is not None):
            # Create log file handler
            try:
                logfile_handler = logging.FileHandler(self.logfile_file)
            except Exception as exception:
                print("Failed to set up log file: %s" % str(exception))
                return False
            # Set log file log level
            try:
                logfile_handler.setLevel(self.logfile_log_level) # only accepts uppercase level names
            except:
                print("Failed to set log file log level: invalid level: '%s'" % self.logfile_log_level)
                return False
            # Create and set formatter, add log file handler to logger
            logfile_formatter = LogFormatter(fmt=self.log_line_template, color=self.logfile_log_color)
            logfile_handler.setFormatter(logfile_formatter)
            logger.addHandler(logfile_handler)
        return logger

    def alter_werkzeug_logger(self):
        werkzeug = logging.getLogger('werkzeug')
        werkzeug.setLevel(logging.ERROR)
        # logging.getLogger('werkzeug').disabled = True
        console_handler = logging.StreamHandler(self.console_log_output)
        requestFilters = RequestIdFilter(server= self.server, service= self.service)
        werkzeug.addFilter(requestFilters)
        # Create and set formatter, add console handler to logger
        console_formatter = LogFormatter(fmt=self.log_line_template, color=self.console_log_color)
        console_handler.setFormatter(console_formatter)
        werkzeug.addHandler(console_handler)
        # if len(werkzeug.handlers) == 1:
        #     formatter = logging.Formatter(self.log_line_template)
        #     werkzeug.handlers[0].setFormatter(formatter)


#!/usr/bin/env python

from setuptools import find_packages, setup

requires = [
    'Flask==2.2.2'
]

setup(
    include_package_data=True,
    name="motifer",
    version='0.0.2',
    description='Generic logs pattern builder for Python and Flask, built on top of logging module.',
    long_description_content_type='text/markdown',
    long_description=open('README.md').read(),
    author='Ankur Mahajan',
    author_email='mahajanankur.nith@gmail.com',
    url='https://www.searchunify.com/',
    packages=find_packages(exclude=['tests*']),
    install_requires=requires,
    license="Apache License",
    python_requires=">= 3",
    project_urls={
        'Documentation': 'https://github.com/mahajanankur/motifer-py/blob/main/README.md',
        'Source': 'https://github.com/mahajanankur/motifer-py',
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
)